{ Bgi Benchmarking program }
{ A Danson 2015 }

{ This program will measure the performance of various BGI based routines
  using a BGI file and mode specified on the command line.

  It is not intended to benchmark the graphics card, although you could
  use it for that purpose. }

program bgibench;

uses graph, time;

var
   driver : string;
   driverString : string;
   mode   : integer;
   modeString : string;
   maxX   : integer;
   maxY   : integer;
   maxColour : integer;
   maxMode : integer;
   maxModeString : string;
   sprites : real;
   vlines  : real;
   hlines  : real;
   rlines  : real;
   circles : real;
   fboxes  : real;
   pboxes  : real;
   pixels  : real;
   btext   : real;
   stext   : real;
   littfhandle : integer;

procedure initdisplay(d : string; m : integer);
var
   driverh:integer;
   gr : integer;
   i : integer;
begin
     for i := 1 to Length(d) do
         d[i] := UpCase(d[i]);
     driver := d;

     { the graph unit does some detection support for these drivers
       so I've used their normal driver handle rather than loading the
       bgi manually }
     if (driver = 'CGA') then driverh := 1
     else if (driver = 'DETECT') then driverh := 0
     else if (driver = 'EGAVGA') then driverh := 3
     else
     begin
          driverh := installuserdriver(d, nil);
          mode := m;

          gr := graphResult;
          if gr<>grok then
          begin
               writeln('error loading driver');
               writeln(graphErrorMsg(gr));
               halt(1);
          end;
     end;

     initgraph(driverh, mode, '.');

     gr:= graphresult;
     if gr<>grok then
     begin
          writeln('error starting display');
          writeln(graphErrorMsg(gr));
          halt(1);
     end;

     clearviewport;
     maxX := getMaxX + 1;
     maxY := getMaxY + 1;
     maxColour := getMaxColor + 1;
     maxMode := getMaxMode;
     modeString := getModeName(mode);
     maxModeString := getModeName(maxMode);
     driverString := getDriverName;
end;


procedure bench_sprites;
var
   sprite : pointer;
   size : integer;
   x,y,i :integer;
   count : longint;
   start, stop : single;
   rate : real;
begin
     randomize;
     for x:=0 to 9 do
         for y:= 0 to 9 do
             putPixel(x,y, random(maxColour));

     size := imageSize(0,0,9,9);
     getMem(sprite, size);
     getImage(0,0,9,9,sprite^);

     count:=0;
     start := timer;
     stop := start + 30;
     while (timer<stop) do
     begin
          for i:= 0 to 29 do
              putImage(random(maxX),random(maxY), sprite^, copyput);
          count := count + 30;
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     sprites := rate;
     freeMem(sprite, size);
end;

procedure bench_lines;
var
   i,z,c : integer;
   count : longint;
   start, stop : single;
   rate : real;
begin
     {vertical line test}
     c:=0;
     start := timer;
     stop := start + 30;
     while timer<stop do
     begin
          setColor(c);
          for i:= 0 to 9 do
              for z:= 0 to 19 do
                   line(i+(z*10),0,i+(z*10),200);
          count := count +  200;
          if c=maxColour-1 then c:=0 else inc(c);
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     vlines := rate;

     {horizontal line test}
     c:=0;
     start := timer;
     stop := start + 30;
     while timer<stop do
     begin
          setColor(c);
          for i:= 0 to 9 do
              for z:= 0 to 19 do
                   line(0,i+(z*10),200,i+(z*10));
          count := count +  200;
          if c=maxColour-1 then c:=0 else inc(c);
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     hlines := rate;

     {random line test}
     c:=0;
     start := timer;
     stop := start + 30;
     while timer<stop do
     begin
          setColor(c);
          for i:= 0 to 49 do
              line(random(maxX),random(maxY),random(maxX),random(maxY));
          count := count +  50;
          if c=maxColour-1 then c:=0 else inc(c);
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     rlines := rate;
end;

procedure bench_circles;
var
   i,z,c : integer;
   cx,cy : integer;
   count : longint;
   start, stop : single;
   rate : real;
begin
     {circle test}
     count := 0;
     c:=0;
     cx := maxX div 2;
     cy := maxY div 2;
     start := timer;
     stop := start + 30;
     while timer<stop do
     begin
          for i:= 0 to 49 do
          begin
              setColor( (c+i) mod maxColour );
              circle(cx,cy, (i+20));
          end;
          count := count +  50;
          if c=maxColour-1 then c:=0 else inc(c);
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     circles := rate;
end;

procedure bench_boxes;
var
   i,z,c,o : integer;
   count : longint;
   start, stop : single;
   rate : real;
begin
     {filled boxes test}
     count := 0;
     start := timer;
     stop := start + 30;
     o:=0;
     while timer<stop do
     begin
          for i:= 0 to 19 do
              for c:= 0 to 19 do
              begin
                   z:= (c + o + (i*5)) mod maxColour;
                   setFillStyle(SolidFill, z);
                   bar(i*10,c*10,(i*10)+9,(c*10)+9);
              end;
          inc(o);
          count := count +  400;
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     fboxes := rate;

     {pattern filled boxes test}
     count := 0;
     start := timer;
     stop := start + 30;
     while timer<stop do
     begin
          for i:= 0 to 19 do
              for c:= 0 to 19 do
              begin
                   z:= (c + o + (i*5)) mod maxColour;
                   setFillStyle(HatchFill, z);
                   bar(i*10,c*10,(i*10)+9,(c*10)+9);
              end;
          inc(o);
          count := count +  400;
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     pboxes := rate;
end;

procedure bench_pixels;
var
   i : integer;
   count : longint;
   start, stop : single;
   rate : real;
begin
     {pixel test}
     count := 0;
     start := timer;
     stop := start + 30;
     while timer<stop do
     begin
          for i:= 0 to 399 do
              putpixel(random(maxX),random(maxY),random(maxColour));
          count := count +  400;
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     pixels := rate;
end;

procedure littfont; external;
{$L LITT.OBJ}

procedure bench_text;
var
   i : integer;
   count : longint;
   start, stop : single;
   rate : real;
begin
     littfhandle := registerBGIFont(@littfont);

     setTextStyle(DefaultFont, HorizDir, 1);

     {bitmap font test}
     count := 0;
     start := timer;
     stop := start + 30;
     while timer<stop do
     begin
          for i:= 0 to 99 do
          begin
               setColor(i mod maxColour);
               outtextxy(random(maxX),random(maxY),'Test Text');
          end;
          count := count +  100;
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     btext := rate;

     setTextStyle(littfhandle, HorizDir, 4);

     {bitmap font test}
     count := 0;
     start := timer;
     stop := start + 30;
     while timer<stop do
     begin
          for i:= 0 to 99 do
          begin
               setColor(i mod maxColour);
               outtextxy(random(maxX),random(maxY),'Test Text');
          end;
          count := count +  100;
     end;
     stop := timer;
     rate := count;
     rate := rate / (stop - start);
     stext := rate;

end;

begin
    if paramCount<2 then
    begin
         writeln('Usage:');
         writeln(' BGIBENCH bgifile graphicsmode');
         halt(0);
    end;
    driver := paramStr(1);
    val(paramStr(2), mode, maxX); {using maxX to store an error code}
    if maxX>0 then
    begin
        writeln('please enter a valid number for the mode');
        halt(1);
    end;

    initDisplay(driver,mode);

    bench_sprites;
    bench_lines;
    bench_circles;
    bench_boxes;
    bench_pixels;
    bench_text;

    closeGraph;
    writeln('results for ',driver);
    writeln(' driver string ', driverString);
    writeln(' mode:',mode,':',modeString);
    writeln(' maxMode:',maxMode,':',maxModeString);
    writeln(' resolution ',maxX,'x',maxY);
    writeln(' colours ',maxColour);
    writeln;
    str(sprites:3:3,driver);
    writeln(' sprites/second ',driver);
    str(vlines:3:3,driver);
    writeln(' vertical lines/second ',driver);
    str(hlines:3:3,driver);
    writeln(' horizontal lines/second ',driver);
    str(rlines:3:3,driver);
    writeln(' random lines/second ',driver);
    str(circles:3:3,driver);
    writeln(' circles/second ',driver);
    str(fboxes:3:3,driver);
    writeln(' filled boxes/second ',driver);
    str(pboxes:3:3,driver);
    writeln(' pattern filled boxes/second ',driver);
    str(pixels:3:3,driver);
    writeln(' put-pixels/second ',driver);
    str(btext:3:3,driver);
    writeln(' bitmapped text/second ',driver);
    str(stext:3:3,driver);
    writeln(' stroked text/second ',driver);
end.